
# coding: utf-8

# # Creating and accessing a fit
# This example shows you, how you can easily calculate and visualize a fit to your data

# In[ ]:


import numpy as np
import xarray as xr
import psyplot.project as psy
get_ipython().run_line_magic('matplotlib', 'inline')
get_ipython().run_line_magic('config', 'InlineBackend.close_figures = False')


# First we start with some example data to make a linear regression from the equation
# ``y(x) = 4 * x + 30``

# In[ ]:


x = np.linspace(0, 100)
y = x * 4 + 30 + 50* np.random.normal(size=x.size)
ds = xr.Dataset({'x': xr.Variable(('experiment', ), x),
                 'y': xr.Variable(('experiment', ), y)})
ds


# We can show this input data using the ``lineplot`` plot method from the psy-simple plugin:

# In[ ]:


raw = psy.plot.lineplot(
    ds, name='y', coord='x', linewidth=0, marker='o', legendlabels='raw data', 
    legend='upper left')


# The visualization of the fit is straight forward using the ``linreg`` plot method:

# In[ ]:


fit = psy.plot.linreg(ds, ax=raw.plotters[0].ax, name='y', coord='x', 
                      legendlabels='fit', color='red')
fit.share(raw[0], keys='legend')
fit.show()


# The shaded red area displays the 95% confidence interval. To access the data for the fit, just use the ``plot_data`` attribute:

# In[ ]:


data = fit[0].psy.plotter.plot_data
data[0]


# You see, that there are new attributes, ``rsquared``, ``intercept`` and ``slope``, the characteristics of the fit.
# As always with the dataset attributes in ``psyplot``, you can visualize them, for example in the legend:

# In[ ]:


fit.update(legendlabels='%(yname)s = %(intercept).3g + %(slope).3g * %(xname)s, R$^2$=%(rsquared).3g')
fit.show()


# To improve the fit, we can also force the line to go through a given fix point. For example here, we know, that the
# fit crosses the y-line at 30:

# In[ ]:


fit.update(fix=30)
fit.show()


# That works for any other point as well. E.g. we also know, that the line goes through ``y = 4 * 10 + 30 = 70``:

# In[ ]:


fit.update(fix=[(10, 70)])
fit.show()


# For more informations, look into the formatoptions of the ``regression`` group

# In[ ]:


fit.summaries('regression')


# In[ ]:


psy.close('all')

